#####################################################
# Company      : Riello UPS GmbH                    #
# Author       : Ruediger Brink                     #
# Copyright(c) : Riello UPS GmbH 2013
# Version      : 1.1                                #
# Date         : 09/11/2012                         #
# Internet     : http://www.riello-ups.de           #
# Last modified: 09/12/2016                         #
##################################################### 

$startTime = Get-Date

#####################################################
# If $RUN_TEST is $true then you can start this     #
# script to see what happens. No action is          #
# performed, only a log file is written to $UPS_LOG.#
#                                                   #
# Possible values: $true or $false                  #
#####################################################
#
$RUN_TEST   = $true

#####################################################
# At the end, it's only possible to shutdown        #
# $UPS_VM or $UPS_HOS but not both. Therefore you   #
# must configure, which of them to be shutdown.     #
#                                                   #
# Possible values: $true or $false                  #
#####################################################
#
$SHUT_UPS_HOST = $true

#####################################################
# Insert here the IP number of the vCenter Server   #
# or the ESX/ESXi host                              #
#####################################################
#
$VI_SERVER  = "10.10.10.10"

#####################################################
# If this script running on a virtual machine, then #
# you must insert in the variable $UPS_VM the name  #
# of this VM. Because this system can not be        #
# shutdown.                                         #
#                                                   #
# If this script running on a physically system,    #
# then leave $UPS_VM blank                          #
#####################################################
#
$UPS_VM    = ""

#####################################################
# Please insert here the path of the installation   #
# directory. Including a backslash at the end!      #
#####################################################
#
$UPS_PATH   = "c:\ProgramData\Upsmon\"

#####################################################
# Where is the log file to be written.              #
# If this blank no log file will be written         #
#####################################################
#
$UPS_LOG    = "ups.log"

#####################################################
# $DOWN_LOOP : counter of how often to be tested    #
#              whether VMs are shutdown             #
# $DOWN_SLEEP: delay in seconds for the next test   #
#####################################################
#
$DOWN_LOOP  = 3
$DOWN_SLEEP = 10

####################################################
# DON'T DELETE THIS ARRAY DECLARATION!!!!!!        #
####################################################
#
$VM_STOP = @()

####################################################
# If you want to sends the stop command instead of #
# shutdown you have to configure this part.        #
# Fill array with values                           #
#                                                  #
# array[0] = Server name           [String]        #
# array[1] = Sequence of Shutdown  [Integer]       #
# array[2] = Delay in seconds      [Integer]       #
# e.g.                                             #
# $VM_STOP += ,@("Server name", 0, 10)             #
####################################################
#
#$VM_STOP += ,@("Server 2", 0, 30)
#$VM_STOP += ,@("Server 1", 1, 30)

####################################################
# DON'T DELETE THIS ARRAY DECLARATION!!!!!!        #
####################################################
#
$VM_DEPENDS = @()

####################################################
# If you have dependencies between your VMs then   #
# you have to configure this part.                 #
# Fill array with values                           #
#                                                  #
# array[0] = Server name           [String]        #
# array[1] = Sequence of Shutdown  [Integer]       #
# array[2] = Delay in seconds      [Integer]       #
# e.g.                                             #
# $VM_DEPENDS += ,@("Server name", 0, 10)          #
####################################################
#
#$VM_DEPENDS += ,@("Server 10", 0, 10)
#$VM_DEPENDS += ,@("Server 8",  1, 10)
#$VM_DEPENDS += ,@("Server 5",  2, 10)
#$VM_DEPENDS += ,@("Server 1",  3, 10)

####################################################
# This is the key to encrypt/decrypt your login    #
# data.                                            #
# It must be the same as in "password.ps1"         #
####################################################
#
$VM_KEY     = (3,4,2,3,6,2,1,9,2,3,2,4,3,1,4,9)

####################################################
# DON'T DELETE THIS ARRAY DECLARATION!!!!!!        #
####################################################
#
$HOST_EXCLUDE = @()

####################################################
# If you manage externel hosts in your vCenter and #
# they should not shut down please insert his      #
# hostname in this array                           #
#                                                  #
####################################################
#
#$HOST_EXCLUDE += ,@("host 02")
#$HOST_EXCLUDE += ,@("host 03")

####################################################
# DON'T DELETE THIS ARRAY DECLARATION!!!!!!        #
####################################################
#
$VM_EXCLUDE = @()

####################################################
# If you manage externel hosts in your vCenter and #
# they should not shut down please insert his      #
# VMs name in this array                           #
#                                                  #
####################################################
#
#$VM_EXCLUDE += ,@("vm 15")
#$VM_EXCLUDE += ,@("vm 16")
#$VM_EXCLUDE += ,@("vm 17")

####################################################
# DON'T DELETE THIS ARRAY DECLARATION!!!!!!        #
####################################################
#
$VM_LAST = @()

####################################################
# If you want shutdown VMs at end please insert    #
# VMs name in this array                           #
#                                                  #
####################################################
#
#$VM_LAST += ,@("vm 20")

####################################################
# DON'T DELETE OR CHANGE THIS DECLARATION!!!!!!    #
####################################################
#
$UPS_HOST   = ""

####################################################
# Function : writeLog                              #
# Parameter: text to write and a switch to write a #
#          : new line first                        #
# Used     : Get-Date                              #
# Returns  :                                       #
####################################################
#
function writeLog {
  param([string]$LogText, [switch]$NewLine)

  if ( $UPS_LOG ) {
    $DATUM = Get-Date -uformat %d.%m.%Y" "%T
    if ( $NewLine ) { echo "`n" >>$UPS_PATH$UPS_LOG }
    if ( $LogText ) { echo "$DATUM $LogText" >>$UPS_PATH$UPS_LOG }
  }
}

if (!$VI_SERVER) {
  if ($RUN_TEST) {
    echo "`nPlease insert the IP number of an 'vCenter Server' or 'ESX/ESXi host'`n"
  }
  writeLog -LogText "Please insert the IP number of an 'vCenter Server' or 'ESX/ESXi host'" -NewLine
  writeLog "Program abort!"
  Exit 1
}

if ( !(Test-Path "$($UPS_PATH)login.log") -or !(Test-Path "$($UPS_PATH)user.log") ) {
  if ($RUN_TEST) {
    echo "`nCould not find file 'login.log' or 'user.log'!`nPlease execute 'password.ps1'`n"
  }
  writeLog -LogText "Could not find 'login.log' or 'user.log'!Please execute 'password.ps1'" -NewLine
  writeLog "Program abort!"
  Exit 1
}

####################################################
# The file name for $PASSWD and $USER must be the  #
# same as in "password.ps1".                       #
####################################################
#
$PASSWD     = Get-Content "$($UPS_PATH)login.log" | ConvertTo-SecureString -key $VM_KEY
$PASSWD     = [Runtime.Interopservices.Marshal]::PtrToStringAuto([Runtime.Interopservices.Marshal]::SecureStringToBStr($PASSWD))

$USER       = Get-Content "$($UPS_PATH)user.log" | ConvertTo-SecureString -key $VM_KEY
$USER       = [Runtime.Interopservices.Marshal]::PtrToStringAuto([Runtime.Interopservices.Marshal]::SecureStringToBStr($USER))

Connect-VIServer -Server $VI_SERVER -User $USER -Password $PASSWD

####################################################
# Function : VM_Exists                             #
# Parameter: VM Name                               #
# Used     :                                       #
# Returns  : $true  if exists                      #
#          : $false if not exists                  #
####################################################
#
function VM_Exists ( $VM_NAME ) {
  #Get-VM | ? {$_.PowerState -eq "PoweredOn"} | foreach {
  Get-VM | foreach {
    if ( $_.Name -eq $VM_NAME ) {
      return $true
    }
  }

  return $false
}

####################################################
# Function : Get_Last                              #
# Parameter: array index of $VM_LAST               #
# Used     : $VM_LAST                              #
# Returns  : on found - located array entry        #
#          : otherwise - an empty array            #
####################################################
#
function Get_Last( $NR ) {
  $VM_LAST | foreach {
    if ( VM_Exists ( $_[$NR] ) ) {
      return $_[$NR]
    }
  }

  return @()
}

####################################################
# Function : Test_Depend                           #
# Parameter: Name of VM to test                    #
# Used     : $VM_DEPENDS                           #
# Returns  : $true if $VM is in array $VM_DEPENDS  #
#          : $false if not                         #
####################################################
#
function Test_Depend( $VM ) {
  if ( $VM_DEPENDS.Count -eq 0 ) {
    return $false
  } else {
    $VM_DEPENDS | foreach {
      if ( $_[0] -eq $VM ) {
        return $true
      }
    }

    return $false
  }
}

####################################################
# Function : Get_Depend                            #
# Parameter: array index of $VM_DEPENDS            #
# Used     : $VM_DEPENDS                           #
# Returns  : on found - located array entry        #
#          : otherwise - an empty array            #
####################################################
#
function Get_Depend( $NR ) {
  $VM_DEPENDS | foreach {
    if ( ($_[1] -eq $NR) -and (VM_Exists ( $_[0] )) ) {
      return $_
    }
  }

  return @()
}

####################################################
# Function : Delete_Depend                         #
# Parameter: array of VMs                          #
# Used     : Test_Depend                           #
# Returns  : array of VMs without $VM_DEPENDS      #
####################################################
#
function Delete_Depend( $VMs ) {
  if ( $VM_DEPENDS.Count -eq 0 ) {
    return $VMs
  } else {
    $Ret_VMs = @()

    $VMs | foreach {
      if ( !(Test_Depend( $_ )) ) {
        $Ret_VMs += $_
      }
    }

    return $Ret_VMs
  }
}

####################################################
# Function : Test_Stop                             #
# Parameter: Name of VM to test                    #
# Used     : $VM_STOP                              #
# Returns  : $true if $VM is in array $VM_STOP     #
#          : $false if not                         #
####################################################
#
function Test_Stop( $VM ) {
  if ( $VM_STOP.Count -eq 0 ) {
    return $false
  } else {
    $VM_STOP | foreach {
      if ( $_[0] -eq $VM ) {
        return $true
      }
    }

    return $false
  }
}

####################################################
# Function : Test_VM_Exclude                       #
# Parameter: Name of VM to test                    #
# Used     : $VM_EXCLUDE                           #
# Returns  : $true if $VM is in array $VM_EXCLUDE  #
#          : $false if not                         #
####################################################
#
function Test_VM_Exclude( $VM ) {
  if ( $VM_EXCLUDE.Count -eq 0 ) {
    return $false
  } else {
    $VM_EXCLUDE | foreach {
      if ( $_ -eq $VM ) {
        return $true
      }
    }

    return $false
  }
}

####################################################
# Function : Test_VM_Last                          #
# Parameter: Name of VM to test                    #
# Used     : $VM_LAST                              #
# Returns  : $true if $VM is in array $VM_LAST     #
#          : $false if not                         #
####################################################
#
function Test_VM_Last( $VM ) {
  if ( $VM_LAST.Count -eq 0 ) {
    return $false
  } else {
    $VM_LAST | foreach {
      if ( $_ -eq $VM ) {
        return $true
      }
    }

    return $false
  }
}

####################################################
# Function : Get_Stop                              #
# Parameter: array index of $VM_STOP               #
# Used     : $VM_STOP                              #
# Returns  : on found - located array entry        #
#          : otherwise - an empty array            #
####################################################
#
function Get_Stop( $NR ) {
  $VM_STOP | foreach {
    if ( ($_[1] -eq $NR) -and (VM_Exists ( $_[0] )) ) {
      return $_
    }
  }

  return @()
}

####################################################
# Function : Delete_Stop                           #
# Parameter: array of VMs                          #
# Used     : Test_Stop                             #
# Returns  : array of VMs without $VM_STOP         #
####################################################
#
function Delete_Stop( $VMs ) {
  if ( $VM_STOP.Count -eq 0 ) {
    return $VMs
  } else {
    $Ret_VMs = @()

    $VMs | foreach {
      if ( !(Test_Stop( $_ )) ) {
        $Ret_VMs += $_
      }
    }

    return $Ret_VMs
  }
}

####################################################
# Function : Delete_VM_Exclude                     #
# Parameter: array of VMs                          #
# Used     : Test_VM_Exclude                       #
# Returns  : array of VMs without $VM_EXCLUDE      #
####################################################
#
function Delete_VM_Exclude( $VMs ) {
  if ( $VM_EXCLUDE.Count -eq 0 ) {
    return $VMs
  } else {
    $Ret_VMs = @()

    $VMs | foreach {
      if ( !(Test_VM_Exclude( $_ )) ) {
        $Ret_VMs += $_
      }
    }

    return $Ret_VMs
  }
}

#######################################################
# Function : Test_Host_Exclude                        #
# Parameter: Name of Host to test                     #
# Used     : $HOST_EXCLUDE                            #
# Returns  : $true if $Host is in array $HOST_EXCLUDE #
#          : $false if not                            #
#######################################################
#
function Test_Host_Exclude( $MY_Host ) {
  if ( $HOST_EXCLUDE.Count -eq 0 ) {
    return $false
  } else {
    $HOST_EXCLUDE | foreach {
      if ( $MY_Host -eq $_ ) {
        return $true
      }
    }

    return $false
  }
}

####################################################
# Function : Delete_Host_Exclude                   #
# Parameter: array of Hosts                        #
# Used     : Test_Host                             #
# Returns  : array of Hosts without $HOST_EXCLUDE  #
####################################################
#
function Delete_Host_Exclude( $MY_Hosts ) {
  if ( $HOST_EXCLUDE.Count -eq 0 ) {
    return $MY_Hosts
  } else {
    $Ret_Hosts = @()

    $MY_Hosts | foreach {
      if ( !(Test_Host_Exclude( $_.Name )) ) {
        $Ret_Hosts += $_
      }
    }

    return $Ret_Hosts
  }
}

####################################################
# Function : Delete_VM_Last                        #
# Parameter: array of VMs                          #
# Used     : Test_VM_Last                          #
# Returns  : array of VMs without $VM_LAST         #
####################################################
#
function Delete_VM_Last( $VMs ) {
  if ( $VM_LAST.Count -eq 0 ) {
    return $VMs
  } else {
    $Ret_VMs = @()

    $VMs | foreach {
      if ( !(Test_VM_Last( $_ )) ) {
        $Ret_VMs += $_
      }
    }

    return $Ret_VMs
  }
}

if ( $RUN_TEST ) {
  writeLog -LogText "Start............: ESXshutdown.ps1 [TEST MODE]" -NewLine
} else {
  writeLog -LogText "Start............: ESXshutdown.ps1 [REAL MODE]" -NewLine
}

if ( $UPS_VM ) {
  ## Get the name of the host where the VM is running
  $UPS_HOST = (Get-VMHost -VM (Get-VM | ? {$_.Name -eq $UPS_VM})).Name
}

if ( $UPS_HOST ) {
  writeLog -LogText "UPS Host is......: $UPS_HOST" -NewLine
} else {
  writeLog -LogText "UPS Host is......: not defined" -NewLine
  $SHUT_UPS_HOST = $false
}

writeLog -LogText ">>> Overall, I found the following HOSTs.." -NewLine
Get-VMHost | ForEach-Object {
  if ( ( !$UPS_HOST ) -or ( $($_.Name) -ne $UPS_HOST )) {
    writeLog("Found HOST.......: $($_.PowerState)`t$($_.Name)")
  } else {
    writeLog("Found HOST (UPS).: $($_.PowerState)`t$($_.Name)")
  }
}

writeLog -LogText ">>> Overall, I found the following VMs.." -NewLine
Get-VM | ForEach-Object {
  if ( ( !$UPS_VM ) -or ( $($_.Name) -ne $UPS_VM )) {
    if ( Test_Depend( $($_.Name) )) {
      writeLog("Found VM.(dep.)..: $($_.PowerState)`t$($_.Name)")
    } else {
      if ( Test_Stop( $($_.Name) )) {
        writeLog("Found VM.(stop)..: $($_.PowerState)`t$($_.Name)")
      } else {
        writeLog("Found VM.........: $($_.PowerState)`t$($_.Name)")
      }
    }
  } else {
    if ( ( $UPS_VM ) -and ( $($_.Name) -eq $UPS_VM )) {
      writeLog("Found VM (UPS)...: $($_.PowerState)`t$($_.Name)")
	} else {
      writeLog("VM (UPS).........: not defined")
	}
  }
}

$VMs = Delete_VM_Last( Delete_VM_Exclude( Delete_Depend( Delete_Stop( Get-VM | ? {$_.Name -ne $UPS_VM} | ? {$_.PowerState -eq "PoweredOn"} ) ) ) )
if ( $VMs ) {
  writeLog -LogText '>>> Shutdown all VMs which are "PoweredOn" without UPS, stop and dependents VMs' -NewLine
  
  foreach ($VM in $VMs ) {
    ## are the vmware-tools installed and started?
    if ( Get-VMGuest $VM | ? {$_.State -eq "Running"} ) {
      ## yes
      if ( !$RUN_TEST ) {
        writeLog("Execute shutdown.: $VM")
        Get-VM $VM | Shutdown-VMGuest -Confirm:$false -Verbose:$false | Out-Null
      } else {
        writeLog("Simulate shutdown: $VM")
      }
    } else {
      ## no
      if ( !$RUN_TEST ) {
        writeLog("Execute stop.....: $VM")
        Stop-VM -VM $VM -Confirm:$false -Verbose:$false | Out-Null
      } else {
        writeLog("Simulate stop....: $VM")
      }
    }
  }
}

if ( $VM_STOP.Count -gt 0 ) {
  writeLog -LogText ">>> Stop now following VMs.." -NewLine;

  for ( $i=0; $i -lt $($VM_STOP.Count); $i++ ) {
    $VM_Get = Get_Stop( $i )

    if ( $($VM_Get.Count) -gt 0 ) {
      if ( ( Get-VM $($VM_Get[0]) | ? {$_.PowerState -eq "PoweredOn"} ) ) {
        if ( !$RUN_TEST ) {
          writeLog("Execute stop.....: $($VM_Get[0])")
          Stop-VM -VM $($VM_Get[0]) -Confirm:$false -Verbose:$false | Out-Null
          if ( $($VM_Get[2]) -gt 0 ) {
            writeLog("Wait.............: $($VM_Get[2]) seconds")
            Start-Sleep -s $($VM_Get[2])
          }
        } else {
          writeLog("Simulate stop....: $($VM_Get[0])")
          if ( $($VM_Get[2]) -gt 0 ) {
            writeLog("Simulate wait....: $($VM_Get[2]) seconds")
          }
        }
      }
    } else {
      writeLog("Error............: VM with Sequence `"$i`" not exists")
    }
  }
}

if ( $VM_DEPENDS.Count -gt 0 ) {
  writeLog -LogText ">>> Shutdown now all the dependents VMs.." -NewLine;

  for ( $i=0; $i -lt $($VM_DEPENDS.Count); $i++ ) {
    $VM_Get = Get_Depend( $i )

    if ( $($VM_Get.Count) -gt 0 ) {
      if ( ( Get-VM $($VM_Get[0]) | ? {$_.PowerState -eq "PoweredOn"} ) ) {
        if ( ( Get-VMGuest $($VM_Get[0]) | ? {$_.State -eq "Running"} ) ) {
          if ( !$RUN_TEST ) {
            writeLog("Execute shutdown.: $($VM_Get[0])")
            Get-VM $($VM_Get[0]) | Shutdown-VMGuest -Confirm:$false -Verbose:$false | Out-Null
            if ( $($VM_Get[2]) -gt 0 ) {
              writeLog("Wait.............: $($VM_Get[2]) seconds")
              Start-Sleep -s $($VM_Get[2])
            }
          } else {
            writeLog("Simulate shutdown: $($VM_Get[0])")
            if ( $($VM_Get[2]) -gt 0 ) {
              writeLog("Simulate wait....: $($VM_Get[2]) seconds")
            }
          }
        } else {
          if ( !$RUN_TEST ) {
            writeLog("Execute stop.....: $($VM_Get[0])")
            Stop-VM -VM $($VM_Get[0]) -Confirm:$false -Verbose:$false | Out-Null
            if ( $($VM_Get[2]) -gt 0 ) {
              writeLog("Wait.............: $($VM_Get[2]) seconds")
              Start-Sleep -s $($VM_Get[2])
            }
          } else {
            writeLog("Simulate stop....: $($VM_Get[0])")
            if ( $($VM_Get[2]) -gt 0 ) {
              writeLog("Simulate wait....: $($VM_Get[2]) seconds")
            }
          }
        }
      }
    } else {
      writeLog("Error............: VM with Sequence `"$i`" not exists")
    }
  }
}

if ( $VM_LAST.Count -gt 0 ) {
  writeLog -LogText ">>> Shutdown now VM_LAST VMs.." -NewLine;

  for ( $i=0; $i -lt $($VM_LAST.Count); $i++ ) {
    $VM_Get = Get_Last( $i )

    if ( $($VM_Get.Count) -gt 0 ) {
      if ( ( Get-VM $VM_Get | ? {$_.PowerState -eq "PoweredOn"} ) ) {
        if ( ( Get-VMGuest $VM_Get | ? {$_.State -eq "Running"} ) ) {
          if ( !$RUN_TEST ) {
            writeLog("Execute shutdown.: $($VM_Get)")
            Get-VM $VM_Get | Shutdown-VMGuest -Confirm:$false -Verbose:$false | Out-Null
          } else {
            writeLog("Simulate shutdown: $($VM_Get)")
          }
        } else {
          if ( !$RUN_TEST ) {
            writeLog("Execute stop.....: $($VM_Get)")
            Stop-VM -VM $VM_Get -Confirm:$false -Verbose:$false | Out-Null
          } else {
            writeLog("Simulate stop....: $($VM_Get)")
          }
        }
      }
    } else {
      writeLog("Error............: VM with Sequence `"$i`" not exists")
    }
  }
}

#
# Give VMs at least 30 seconds time to shutdown
#
#writeLog -LogText ">>> Wait 30 seconds, to give all VMs time to shutdown.." -NewLine
#Start-Sleep -s 30

## Loop to wait for the shutdown of all VMs
## max $DOWN_LOOP times 
writeLog -LogText ">>> Now we test whether all systems are down.." -NewLine;
$VMs = Delete_VM_Exclude( Get-VM | ? {$_.Name -ne $UPS_VM} | ? {$_.PowerState -eq "PoweredOn"} )
while ( ( $VMs ) -and ( $DOWN_LOOP -gt 0 )) {
  #writeLog("DOWN_LOOP.......: $DOWN_LOOP")
  if ( $DOWN_LOOP -gt 0 ) {
    foreach ( $VM in $VMs ) {
      writeLog("Still alive......: $VM")
    }
    writeLog("Wait.............: $DOWN_SLEEP seconds")
    Start-Sleep -s $DOWN_SLEEP
    $VMs = Get-VM | ? {$_.Name -ne $UPS_VM} | ? {$_.PowerState -eq "PoweredOn"}
  }
  $DOWN_LOOP -= 1
}

if ( $DOWN_LOOP -gt 0 ) {
  writeLog -LogText ">>> All VMs are down!" -NewLine
} else {
  $VMs = Delete_VM_Exclude( Get-VM | ? {$_.Name -ne $UPS_VM} | ? {$_.PowerState -eq "PoweredOn"} )
  writeLog -LogText ">>> This VMs are now still alive and therefore we send the stop command.." -NewLine
  foreach ( $VM in $VMs ) {
    if ( !$RUN_TEST ) {
      writeLog("Execute stop.....: $VM")
      Stop-VM -VM $VM -Confirm:$false -Verbose:$false | Out-Null
    } else {
      writeLog("Simulate stop....: $VM")
    }
  }
}

writeLog -LogText ">>> Shutdown all ESX hosts without UPS Host: $UPS_HOST`n" -NewLine
## if $UPS_HOST not exist, then shutdown all hosts
Delete_Host_Exclude( Get-VMHost | ? {$_.Name -ne $UPS_HOST} | ? {$_.PowerState -eq "PoweredOn"} ) | ForEach-Object {
  if ( !$RUN_TEST ) {
    writeLog("Execute shutdown.: $_")
    Get-View $_.ID | Foreach {$_.ShutdownHost_Task($TRUE)} | Out-Null
  } else {
    writeLog("Simulate shutdown: $_")
  }
}

if ( $SHUT_UPS_HOST -and $UPS_HOST ) {
  writeLog -LogText ">>> At last shutdown UPS Host: $UPS_HOST" -NewLine
  ## If $UPS_HOST exist, then shutdown it at last 
  Get-VMHost | ? {$_.Name -eq $UPS_HOST} | ForEach-Object {
    if ( !$RUN_TEST ) {
      writeLog("Execute shutdown.: $_")
      Get-View $_.ID | Foreach {$_.ShutdownHost_Task($TRUE)} | Out-Null
    } else {
      writeLog("Simulate shutdown: $_")
    }
  }
}

Disconnect-VIServer -Confirm:$false

$endTime = Get-Date

writeLog -LogText "Total elapsed time: $(( $endTime - $startTime ).TotalSeconds) seconds`n" -NewLine
